<?php
	/**
	  * PaidInform
	  *
	  * @author Somprasong Kulasatitporn / IamPing.com
	  * @copyright IamPing.com / PrestaShop
	  * @license http://www.opensource.org/licenses/osl-3.0.php Open-source licence 3.0
	  * @version 0.1
	  * @Edit By Jakkritch Pen-Ak / www.PrestaLights.com
	  * @version 0.2
	  * @Edit By Nokaek Shopping /www.nokaek.com
	  */

	/*
		Updated

		v 0.2.7
		[*] BO : Copy icon to order status, change new icon

		v 0.2.6
		[*] FO : Fix incorrect order number id

		v 0.2.5
		[*] BO : Clean
		[*] FO : Move file payment-form.php to root

		v 0.2.4
		[*] BO : Fix update setting has error
		[*] BO : Fix install w/o create duplicate order status
		[*] BO : Fix remove hook myAccount after uninstall

		v 0.2.3
		[*] BO : change hook position form top to blockmyaccount
		[+] BO : Link bank infomation with module bankwirethai
	*/
	class PaidInform extends Module
	{
		private $errors		= array();
		private $formData	= array();
		private $id_lang	= 0;

		public function __construct()
		{
			$this->name = 'paidinform';
			$this->tab = 'JKO SHOP modules';
			$this->version = '0.2.8';
			$this->full_dir = _PS_MODULE_DIR_.$this->name.'/';
			$this->full_url = _MODULE_DIR_.$this->name.'/';
			$this->new_status_name = 'Paid Informed';

			$this->new_status  = Configuration::get('STATUS_AFTER_MANUAL_PAID');
			$this->alert_admin = Configuration::get('ALERT_AFTER_MANUAL_PAID');

			$this->id_lang = intval(Configuration::get('PS_LANG_DEFAULT'));

			parent::__construct(); /* The parent construct is required for translations */

			$this->page = basename(__FILE__, '.php');
			$this->displayName = $this->l('Order Paid Inform form');
			$this->description = $this->l('A form to accept a paid information from customer');
			$this->confirmUninstall = $this->l('Are you sure you want to delete your details ?');
		}

		public function install()
		{
			$new_status = $this->createNewStatus();
			if (!$new_status)
				return FALSE;
			
			$this->createConfig($new_status);

			// crate payment-form.php at root
			$content = '<?php
include_once(\'./config/config.inc.php\');
include_once(\'./header.php\');
include_once(\'./modules/'.$this->name.'/paidinform.php\');
$paidInform = new PaidInform();
$paidInform->processPaidInform();
include_once(\'./footer.php\');
?>';
			
			$handle = fopen(_PS_ROOT_DIR_.'/payment-form.php', "w");
			fwrite($handle, $content);
			fclose($handle);

			if (!parent::install() OR !$this->registerHook('myAccountBlock'))
				return false;
				
			return TRUE;
		}

		public function uninstall()
		{
			if (file_exists(_PS_ROOT_DIR_.'/payment-form.php'))
			{ 
				if (!unlink(_PS_ROOT_DIR_.'/payment-form.php'))
				{
					return false;
				}
			}

			if (!parent::uninstall() OR 
				!Configuration::deleteByName('STATUS_AFTER_MANUAL_PAID') OR
				!Configuration::deleteByName('ALERT_AFTER_MANUAL_PAID') OR
				!Configuration::deleteByName('BANK_ACCOUNT') OR
				!Configuration::deleteByName('BANK_PAYMENT_METHOD')
				)
				return false;
				
			return TRUE;
		}

		function createConfig($new_status)
		{
			$configObj = new Configuration();
			$configObj->name = 'STATUS_AFTER_MANUAL_PAID';
			$configObj->value = $new_status;
			$configObj->add();
			
			$configObj->name = 'ALERT_AFTER_MANUAL_PAID';
			$configObj->value = 1;
			$configObj->add();
		}

		// Auto generate a new status to change when customer already submit the paid information form
		function createNewStatus()
		{
			$defaultLanguage = new Language($this->id_lang);
			$_POST['name_'.$defaultLanguage->id] = $this->new_status_name;
			
			$_POST['logable'] = 1;
			$_POST['color']   = '#CCCCCC';
			$_POST['submitAddorder_state'] = 1;		

			// check duplicate order state
			$os = Db::getInstance()->getRow('SELECT * 
				FROM `'._DB_PREFIX_.'order_state_lang` 
				WHERE `id_lang`='.$this->id_lang.' 
					AND `name` = "'.$this->new_status_name.'"');

			if (empty($os))
			{
				include_once(PS_ADMIN_DIR.'/tabs/AdminOrdersStates.php');
				$adminStatus = new AdminOrdersStates();
				$orderStatus = new $adminStatus->className();
				$this->copyFromPost($orderStatus, $adminStatus->table);
				if ($orderStatus->add())
				{
					// Do copy bank icon
					$this->copyImageStatus($orderStatus->id);

					return $orderStatus->id;
				}
				else 
					return FALSE;
			}
			else
			{
				// Do copy bank icon
				$this->copyImageStatus($os['id_order_state']);

				return $os['id_order_state'];
			}
		}

		// Copy status icon to order status
		function copyImageStatus($imageStatusId)
		{
			// Do copy bank icon
			if (!file_exists(_PS_IMG_DIR_.'os/'.$imageStatusId.'.gif'))
			{
				if (file_exists(_PS_ROOT_DIR_.'/modules/'.$this->name.'/logo.gif'))
				{
					if (!copy(_PS_ROOT_DIR_.'/modules/'.$this->name.'/logo.gif', _PS_IMG_DIR_.'os/'.$imageStatusId.'.gif'))
						echo 'cannot copy bank logo.';
				}
			}

			// Do copy bank icon 
			if (!file_exists(_PS_IMG_DIR_.'tmp/order_state_mini_'.$imageStatusId.'.gif'))
			{
				if (file_exists(_PS_ROOT_DIR_.'/modules/'.$this->name.'/logo.gif'))
				{
					if (!copy(_PS_ROOT_DIR_.'/modules/'.$this->name.'/logo.gif', _PS_IMG_DIR_.'tmp/order_state_mini_'.$imageStatusId.'.gif'))
						echo 'cannot copy bank logo.';
				}
			}		
		}

		// copied from class/AdminTab.php
		protected function copyFromPost(&$object, $table)
		{
			/* Classical fields */
			foreach ($_POST AS $key => $value)
				if (key_exists($key, $object) AND $key != 'id_'.$table)
					$object->{$key} = $value;

			/* Multilingual fields */
			$rules = call_user_func(array('OrderState', 'getValidationRules'), get_class($object));
			if (sizeof($rules['validateLang']))
			{
				$languages = Language::getLanguages();
				foreach ($languages AS $language)
					foreach ($rules['validateLang'] AS $field => $validation)
						if (isset($_POST[$field.'_'.intval($language['id_lang'])]))
							$object->{$field}[intval($language['id_lang'])] = $_POST[$field.'_'.intval($language['id_lang'])];
			}
		}

		public function hookMyAccountBlock($params)
		{
			return '<li id="header_link_paidinform"><a href="'.__PS_BASE_URI__.'payment-form.php">'.$this->l('Paid inform').'</a></li>';
		}
		
		function processPaidInform()
		{
			global $smarty, $cookie;
			if (Tools::isSubmit('submitPaidForm'))
			{
				$this->errors = $this->_paidFormValidation();
				if (!$this->errors) 
				{
					$message = new Message();
					$message->message  = htmlentities($this->makeOrderMessage(), ENT_COMPAT, 'UTF-8');
					$message->id_order = $this->formData['id_order'];
					$message->private  = 0;

					if ($message->add()) 
					{
						$history = new OrderHistory();
						$history->id_order = $this->formData['id_order'];
						$history->changeIdOrderState($this->new_status, $this->formData['id_order']);

						if ($history->addWithemail(true))
						{
							if($this->alert_admin) 
							{
								$this->getShopContactEmail();
								$templVar = array(
									'{id_order}' => sprintf("#%06d",$this->formData['id_order']),
									'{bank}'     => $this->formData['bank'], 
									'{payment}'  => $this->formData['paymentmethod'], 
									'{amount}'   => $this->formData['amount'], 
									'{date}'     => $this->formData['date'], 
									'{time}'     => $this->formData['time'], 
									'{email}'   => $this->formData['email'],
									'{customer}'   => $this->formData['name'],
									'{telephone}'   => $this->formData['telephone'],
									'{remark}'   => $this->formData['remark']
								);

								$send = Mail::Send(
									$this->id_lang, //id_lang
									'alert_admin', //template
									'Paid Informed for order# '.$history->id_order, //subject
									$templVar, //templateVars
									$this->getShopContactEmail(), //to
									NULL, //toName
									NULL, //from
									NULL, //fromName
									NULL, //fileAttachment
									NULL, //modeSMTP
									$this->full_dir.'mail/' //templatePath								
								);
                                                                
								if ($send)
								{
									$smarty->assign('success', 1);
								}
								else
								{
									$this->errors[] = Tools::displayError('an error occurred while sending message');
								}
							}
							else
							{
								$smarty->assign('success', 1);
							}
						}
						else 
						{
							$this->errors[] = Tools::displayError('an error occurred while chaning order status');
						}
					}
					else 
					{
						$this->errors[] = Tools::displayError('an error occurred while saving paid information');
					}
				}
			}
			$smarty->assign('module_name', $this->name);
			//$smarty->assign('calendar_script', $this->_displayCalendar());
                        $smarty->assign('lang', Db::getInstance()->getValue('SELECT iso_code FROM '._DB_PREFIX_.'lang WHERE `id_lang` = '.intval($cookie->id_lang)));
			$smarty->assign('errors', $this->errors);
			$smarty->assign('fullname', $cookie->customer_firstname.' '.$cookie->customer_lastname);
			$smarty->assign('email', $cookie->email);
			$smarty->assign('full_module_dir', $this->full_dir);

			$bank_accounts = array();
			
			// Check existing and active
			$bankwire = Module::getInstanceByName('bankwirethai');

			if (Validate::isLoadedObject($bankwire))
			{
				if ($bankwire->active)
				{
					foreach($bankwire->banks->getActive() as $bankobj)
					{
						$bank_accounts[] = $bankobj['bank_name'].' '.$bankobj['acnumber'];
					}				
				}
			}
			
			if (sizeof($bank_accounts) == 0)
			{
				$bank_accounts = explode('|', Configuration::get('BANK_ACCOUNT'));
			}

			$smarty->assign('bank_accounts', $bank_accounts);	
			$smarty->assign('payment_methods', explode('|', Configuration::get('BANK_PAYMENT_METHOD')));

			echo $this->display(__FILE__, 'paid-form.tpl');
		}
		
		function getShopContactEmail()
		{
			$contact = new Contact(intval(2), intval($cookie->id_lang));
			if($contact AND $contact->email) {
				return $contact->email;
			}
			else {
				$contact = new Contact(intval(1), intval($cookie->id_lang));
				return $contact->email;
			}
		}
		
		function makeOrderMessage()
		{
			$message = "\nThe order has been paid with below information\n";
			if (isset($this->formData['date']) AND $this->formData['date'] AND isset($this->formData['time']) AND $this->formData['time']) {
				$message .= "Paid Date: ".$this->formData['date'] .' '.$this->formData['time']."\n";
			}
			if (isset($this->formData['name']) AND $this->formData['name']) {
				$message .= "Name: ".$this->formData['name']."\n";
			}
			if (isset($this->formData['email']) AND $this->formData['email']) {
				$message .= "Email: ".$this->formData['email']."\n";
			}
			if (isset($this->formData['telephone']) AND $this->formData['telephone']) {
				$message .= "Telephone: ".$this->formData['telephone']."\n";
			}
			if (isset($this->formData['bank']) AND $this->formData['bank']) {
				$message .= "Bank information: ".$this->formData['bank']."\n";
			}
			if (isset($this->formData['paymentmethod']) AND $this->formData['paymentmethod']) {
				$message .= "Payment Method: ".$this->formData['paymentmethod']."\n";
			}
			if (isset($this->formData['amount']) AND $this->formData['amount']) {
				$message .= "Paid amount: ".$this->formData['amount']."\n";
			}
			if (isset($this->formData['remark']) AND $this->formData['remark']) {
				$message .= "Remark: ".$this->formData['remark']."\n";
			}
			
			return $message;
		}
		
		function _displayCalendar()
		{
			return 
				'<link  href="'.$this->full_url.'/css/calendar_style.css" rel="stylesheet" type="text/css" />
				<SCRIPT type="text/javascript" language="Javascript" SRC="'.$this->full_url.'/js/epoch_classes.js"></SCRIPT>
				<script type="text/javascript">
					var bas_cal, dp_cal, ms_cal;      
					window.onload = function () {
					  dp_cal = new Epoch("epoch_popup", "popup", document.getElementById("date"));
					};
				</script>
			';
		}
		
		private function _paidFormValidation()
		{
			$errors = array();
			if (!($this->formData['email'] = Tools::getValue('email')) OR !Validate::isEmail($this->formData['email']))
				$errors[] = Tools::displayError('invalid e-mail address');
			elseif (!($this->formData['telephone'] = nl2br2(Tools::getValue('telephone'))))
				$errors[] = Tools::displayError('telephone cannot be blank');
			elseif (!($this->formData['amount'] = nl2br2(Tools::getValue('amount'))))
				$errors[] = Tools::displayError('amount cannot be blank');
			elseif (!($this->formData['name'] = nl2br2(Tools::getValue('name'))))
				$errors[] = Tools::displayError('Name cannot be blank');
			elseif (!($this->formData['id_order'] = nl2br2(Tools::getValue('id_order'))))
				$errors[] = Tools::displayError('id_order cannot be blank');
			elseif (!($this->formData['date'] = nl2br2(Tools::getValue('date'))))
				$errors[] = Tools::displayError('date cannot be blank');
			elseif (!($this->formData['time'] = nl2br2(Tools::getValue('Time_Hour').':'.Tools::getValue('Time_Minute'))))
				$errors[] = Tools::displayError('time cannot be blank');
			elseif (!($this->formData['bank'] = nl2br2(Tools::getValue('bank'))))
				$errors[] = Tools::displayError('bank cannot be blank');
			elseif (!($this->formData['paymentmethod'] = nl2br2(Tools::getValue('paymentmethod'))))
				$errors[] = Tools::displayError('payment method cannot be blank');
			else
			{
				$this->formData['remark'] = nl2br2(Tools::getValue('remark'));
				
				// Allow only numbers
				if ($id_order = preg_replace('/[^0-9]/', '', $this->formData['id_order']))
				{
					$this->formData['id_order'] = !empty($id_order) ? intval($id_order) : 0;
				}

				$order = new Order($this->formData['id_order']);

				if (!($order->id_cart) OR !($order->id_customer)) 
					$errors[] = Tools::displayError('order number is not exist');
				else {
					$custObj = new Customer();
					$customer = $custObj->getByEmail($this->formData['email']);
					if (!($customer) OR $customer->id != $order->id_customer) 
						$errors[] = Tools::displayError("Email doesn't match with email in order");
				}
			}

			return $errors;
		}

		private function _postProcess()
		{
			Configuration::updateValue('ALERT_AFTER_MANUAL_PAID', Tools::getValue('alert_admin'));
			
			$bankwire = Module::getInstanceByName('bankwirethai');

			if (Validate::isLoadedObject($bankwire))
			{
				if ($bankwire->active)
				{
					$bank_accounts = array();

					// Remove empty data from array
					foreach(Tools::getValue('bank_account', array()) as $bank_account)
					{
						if ($bank_account != '')
						{
							array_push($bank_accounts, $bank_account);
						}
					}

					if (Tools::getValue('bank_account_new', '') != '')
					{
						array_push($bank_accounts, Tools::getValue('bank_account_new'));
					}	
					
					Configuration::updateValue('BANK_ACCOUNT', implode('|', $bank_accounts));
				}
			}

			$payment_methods = array();

			// Remove empty data from array
			foreach(Tools::getValue('payment_method', array()) as $payment_method)
			{
				if ($payment_method != '')
				{
					array_push($payment_methods, $payment_method);
				}
			}

			if (Tools::getValue('payment_method_new', '') != '')
			{
				array_push($payment_methods, Tools::getValue('payment_method_new'));
			}

			Configuration::updateValue('BANK_PAYMENT_METHOD', implode('|', $payment_methods));

			return '<div class="conf confirm"><img src="../img/admin/ok.gif" alt="'.$this->l('ok').'" /> '.$this->l('Settings updated').'</div>';
		}

		private function _displayInfo()
		{
			$html .= '<img src="../img/admin/gold.gif" style="float:left; margin-right:15px;"><b>'.$this->l('This module allows client to fill their paid information when they use a manual payment method.').'</b><br /><br />';
			$html .= $this->l('The example of the manaul payment are Bank wire, Internet Banking, Deposit Bank, etc').'<br />';
			$html .= $this->l('This module will set the order status to Paid Informed.').'<br /><br />';
			return $html;
		}

		private function _displayForm()
		{
			global $smarty;

			$bank_accounts = Configuration::get('BANK_ACCOUNT') === false ? 
									array() : 
									explode('|', Configuration::get('BANK_ACCOUNT'));

			$payment_methods = Configuration::get('BANK_PAYMENT_METHOD') === false ? 
									array() : 
									explode('|', Configuration::get('BANK_PAYMENT_METHOD'));

			$smarty->assign('uri', $_SERVER['REQUEST_URI']);
			$smarty->assign('checked', $this->alert_admin ? 'checked' : '');
			$smarty->assign('bank_accounts', $bank_accounts);
			$smarty->assign('payment_methods', $payment_methods);

			// Hide bank accounts setting when bankwirethai installed and active
			$smarty->assign('withbankwirethai', Module::getInstanceByName('bankwirethai')->active);

			return $this->display(__FILE__, 'paid-inform-configure.tpl');
		}

		public function getContent()
		{
			$html = '<h2>'.$this->displayName.'</h2>';

			if (Tools::getValue('btnSubmit'))
			{
				$html .= $this->_postProcess();
			}
			else
			{
				$html .= '<br />';
			}

			$html .= $this->_displayInfo();
			$html .= $this->_displayForm();

			return $html;
		}
	}
?>